/**
 *  Copyright (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  Licensed under the GPL, Version 3.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/gpl.html
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * --------------------------------------------------------------------------
 *  版权 (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  本开源软件遵循 GPL 3.0 协议;
 *  如果您不遵循此协议，则不被允许使用此文件。
 *  你可以从下面的地址获取完整的协议文本
 *
 *       http://www.gnu.org/licenses/gpl.html
 */
package org.tinygroup.tree;

import java.util.List;

/**
 * 2011-5-25增加getSubPaths方法
 */
/**
 * 
 * @author luoguo
 * 
 * @param <T>代表路径的对象
 * @param <E>
 *            代表数据的对象
 */
public interface Tree<T, E> {
	/**
	 * 获得父节点
	 * 
	 * @return
	 */
	Tree<T, E> getParent();

	void setParent(Tree<T, E> parent);

	/**
	 * 返回路径
	 * 
	 * @return
	 */
	T getPath();

	/**
	 * 返回完全路径 包括当前结点路径
	 * 
	 * @return
	 */
	List<T> getFullPath();

	/**
	 * 获取所有子树
	 * 
	 * @return
	 */
	List<Tree<T, E>> getSubTrees();

	/**
	 * 返回在子树中是路径名为name的子树
	 * 
	 * @param t
	 * @return
	 */
	Tree<T, E> getSubTree(T path);

	/**
	 * 从当前节点开始，在其子孙中查找指定路径名的子树，哪个先找到返回哪个
	 * 
	 * @param t
	 * @return
	 */
	Tree<T, E> getTree(T path);

	/**
	 * 在当前子树前头增加树 返回所添加的子树
	 * 
	 * @param t
	 * @return
	 */
	Tree<T, E> addTreeOnTail(T path, E data);

	Tree<T, E> addTreeOnTail(Tree<T, E> tree);

	Tree<T, E> addTreeOnTail(T path);

	/**
	 * 在子树结尾增加树 返回所增加的子树
	 * 
	 * @param t
	 * @return
	 */
	Tree<T, E> addTreeOnHead(T path);

	Tree<T, E> addTreeOnHead(T path, E data);

	/**
	 * 在指定元素所在的子树前增加子树
	 * 
	 * @param object
	 * @param newObject
	 * @return
	 */
	Tree<T, E> addTreeBefore(T path, T newPath);

	Tree<T, E> addTreeBefore(T path, Tree<T, E> tree);

	Tree<T, E> addTreeBefore(T path, T newPath, E data);

	Tree<T, E> addTreeAfter(T path, T newPath, E data);

	Tree<T, E> addTreeAfter(T path, Tree<T, E> tree);

	/**
	 * 在指定元素所在的子树后增加子树
	 * 
	 * @param object
	 * @param newObject
	 * @return
	 */
	Tree<T, E> addTreeAfter(T path, T newPath);

	/**
	 * 返回当前节点的数据
	 * 
	 * @return
	 */
	E getData();

	void setData(E data);

	boolean isLeaf();

	/**
	 * 返回子树个数
	 * 
	 * @return
	 */
	int getDegree();

	/**
	 * 返回树高
	 * 
	 * @return
	 */
	int getHeight();

	/**
	 * 从当前结点开始，根据路径返回数据 返回路径最后一个结点树
	 * 
	 * @param path
	 * @return
	 */
	Tree<T, E> getTreeByPath(T[] path);

	/**
	 * 从当前结点开始，根据路径返回数据 如果路径不存在，或数据不存在，则返回null 返回最后一个结点树数据
	 * 
	 * @param path
	 * @return
	 */
	E getDataByPath(T[] path);

	/**
	 * 清除所有子树
	 */
	void clear();

	List<T> getSubPaths();

	/**
	 * 在子树前面增加树
	 * 
	 * @param tree
	 * @return
	 */
	Tree<T, E> addTreeOnHead(Tree<T, E> tree);

	/**
	 * 删除一棵子树
	 * 
	 * @param tree
	 */
	void removeTree(Tree<T, E> tree);

	void removeTree(T path);

	/**
	 * 删除一个路径
	 * 
	 * @param path
	 */
	void removeTree(T[] path);
}
